@@ -29,7 +29,9 @@ class PGFunction(ReplaceableEntity):
     def __init__(self, schema: str, signature: str, definition: str):
         super().__init__(schema, signature, definition)
         # Detect if function uses plpgsql and update escaping rules to not escape ":="
-        is_plpgsql: bool = "language plpgsql" in normalize_whitespace(definition).lower()
+        is_plpgsql: bool = "language plpgsql" in normalize_whitespace(definition).lower().replace(
+            "'", ""
+        )
         escaping_callable = escape_colon_for_plpgsql if is_plpgsql else escape_colon_for_sql
         # Override definition with correct escaping rules
         self.definition: str = escaping_callable(strip_terminating_semicolon(definition))
@@ -66,7 +68,7 @@ class PGFunction(ReplaceableEntity):
         return '"' + name + '"(' + remainder
 
     def to_sql_statement_create(self):
-        """ Generates a SQL "create function" statement for PGFunction """
+        """Generates a SQL "create function" statement for PGFunction"""
         return sql_text(
             f"CREATE FUNCTION {self.literal_schema}.{self.literal_signature} {self.definition}"
         )
@@ -95,7 +97,7 @@ class PGFunction(ReplaceableEntity):
         )
 
     def to_sql_statement_create_or_replace(self):
-        """ Generates a SQL "create or replace function" statement for PGFunction """
+        """Generates a SQL "create or replace function" statement for PGFunction"""
         yield sql_text(
             f"CREATE OR REPLACE FUNCTION {self.literal_schema}.{self.literal_signature} {self.definition}"
         )
@@ -159,7 +161,7 @@ class PGFunction(ReplaceableEntity):
         )
 
         rows = sess.execute(sql, {"schema": schema}).fetchall()
-        db_functions = [PGFunction.from_sql(x[3]) for x in rows]
+        db_functions = [cls.from_sql(x[3]) for x in rows]
 
         for func in db_functions:
             assert func is not None
