@@ -44,7 +44,7 @@ class PGPolicy(OnEntityMixin, ReplaceableEntity):
         raise SQLParseFailure(f'Failed to parse SQL into PGPolicy """{sql}"""')
 
     def to_sql_statement_create(self):
-        """ Generates a SQL "create poicy" statement for PGPolicy """
+        """Generates a SQL "create poicy" statement for PGPolicy"""
 
         return sql_text(f"CREATE POLICY {self.signature} on {self.on_entity} {self.definition}")
 
@@ -105,9 +105,7 @@ class PGPolicy(OnEntityMixin, ReplaceableEntity):
             schema = coerce_to_quoted(schema)
             table = coerce_to_quoted(table)
             policy_name = coerce_to_quoted(policy_name)
-            policy = PGPolicy.from_sql(
-                f"create policy {policy_name} on {schema}.{table} {definition}"
-            )
+            policy = cls.from_sql(f"create policy {policy_name} on {schema}.{table} {definition}")
             db_policies.append(policy)
 
         for policy in db_policies:
