@@ -1,8 +1,7 @@
-from typing import TYPE_CHECKING, Optional, Type
+from typing import TYPE_CHECKING, Type
 
 from alembic.autogenerate import renderers
 from alembic.operations import MigrateOperation, Operations
-from sqlalchemy.orm import Session
 from typing_extensions import Protocol
 
 from alembic_utils.exceptions import UnreachableException
@@ -96,7 +95,8 @@ def drop_entity(operations, operation):
 @Operations.implementation_for(RevertOp)
 def replace_or_revert_entity(operations, operation):
     target: "ReplaceableEntity" = operation.target
-    operations.execute(target.to_sql_statement_create_or_replace())
+    for stmt in target.to_sql_statement_create_or_replace():
+        operations.execute(stmt)
 
 
 ##########
@@ -135,40 +135,19 @@ def render_replace_entity(autogen_context, op):
 def render_revert_entity(autogen_context, op):
     """Collect the entity definition currently live in the database and use its definition
     as the downgrade revert target"""
+    # At the time is call is made, the engine is disconnected
 
-    # Note: There can be differences in the local definition's identity property and the
-    # corresponding record in the database. To avoid that issue, we first retrieve the
-    # local definition's corresponding definition once its been rendered in the db
-    # and then use that identity property to find the existing definition.
-
-    # We should never reach this function call unless there is an existing definition
-    # so failing to find a match results in an unreachable exception
+    # We should never reach this call unless an update's revert is being rendered
+    # In that case, get_required_migration_op  has cached the database's liver version
+    # as target._version_to_replace
 
     target = op.target
     autogen_context.imports.add(target.render_import_statement())
 
-    context = autogen_context
+    db_target = target._version_to_replace
 
-    # The connection autogen_context engine is closed
-    engine = autogen_context.connection.engine
-
-    with engine.connect() as connection:
-        sess = Session(bind=connection)
-
-        db_def = target.get_database_definition(sess)
-
-        db_target: Optional["ReplaceableEntity"] = None
-
-        db_entities = sorted(
-            db_def.from_database(sess, schema=db_def.schema), key=lambda x: x.identity
-        )
-        for existing in db_entities:
-            if existing.identity == db_def.identity:
-                db_target = existing
-                break
-
-        if db_target is None:
-            raise UnreachableException
+    if db_target is None:
+        raise UnreachableException
 
     variable_name = db_target.to_variable_name()
     return db_target.render_self_for_migration() + f"op.replace_entity({variable_name})"
